/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netnames.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          inet addr resolver
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/18
**
** DESCRIPTION:        resolve an internet address/port , if possible
**                     Based on a MRU stack
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETNAMES_C
      	    
#include "netmon.h"
#include "nettcpso.h"	   
#include "netnames.h"


/*
 * GLOBALS
 */

static char    *tcp_proto = "tcp";
static char    *udp_proto = "udp";
NETADDR  mru;
NETPORT  mru_udp;                 /* tcp port list */
NETPORT  mru_tcp;                 /* udp port list */

/*
 * local prototypes
 */

int NetPortResolve( u_short port, char *buf, char *proto );
int NetAddrResolve( u_long addr, char *buf );

/*****************************************************************
** NAME:        NetNameOpen                                 NETMON
** SYNOPSIS:    void NetNameOpen
**              (
**                void
**              )
** RELATED TO:  NetNameClose()
** DESCRIPTION: initialise the port/address resolver
** REMARKS:     - uses MRU cache
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void NetNameOpen( void )
{
    memset( &mru    , 0, sizeof( NETADDR ));
    memset( &mru_udp, 0, sizeof( NETPORT ));
    memset( &mru_tcp, 0, sizeof( NETPORT ));

    mru.max      = MRU_LEN;
    mru_udp.max  = MRU_LEN;
    mru_tcp.max  = MRU_LEN;
}

/*****************************************************************
** NAME:        AddrLookup                                  NETMON
** SYNOPSIS:    void AddrLookup
**              (
**                register u_long addr
**              )
** RELATED TO:  -
** DESCRIPTION: return a pointer to string containing the hostname
** REMARKS:     - used in TCPSO and UDPSO modules
** RETURN TYPE: char *
** EXIT STATUS: NULL : error
**              ?    : ptr to hostname
*****************************************************************/

char *AddrLookup( register u_long addr )
{
    register short       index;
    register NETADDRENT *cur;
             NETADDRENT *new;

    /*
     * search in MRU address cache
     */

    index = 0;
    for ( cur = mru.head; cur != NULL; cur = cur->next )
    {
        ++index;
        if ( addr == cur->addr )
            break;
    }

    /*
     * extract or malloc a new element
     */

    if ( cur == NULL )
    {
        /* not found */

        ++mru.misses;

L0:     if ( mru.len == mru.max )
        {
            new            = mru.tail;
            mru.tail       = new->prev;
            mru.tail->next = NULL;
            mru.len--;
        }
        else
        {
            new = (NETADDRENT *) malloc( sizeof( NETADDRENT ));
            if ( new == NULL )
            {
                if ( mru.len < MRU_TOP )
                    eprintf( "\nMRU_ADDR malloc failure\n" );

                mru.max = mru.len;
                goto L0;
            }
        }

       if ( mru.len == 0 )
       {
           new->prev      = NULL;
           new->next      = NULL;
           mru.head       = new;
           mru.tail       = new;
       }
       else
       {
           new->prev      = NULL;
           new->next      = mru.head;
           mru.head->prev = new;
           mru.head       = new;
       }
       mru.len++;
       new->addr = addr;

       /*
        * resolve the address
        */

       if ( NetAddrResolve( addr, new->name ) < 0 )
          ++mru.nores;
       return new->name;
    }
    else
    {
        /* found */

        ++mru.hits;
        new = cur;

        /*
         * Move a found element to the front position only if its too far
         * away. We have set this position to MRU_TOP.
         */

        if ( index > MRU_TOP )
        {
            /*
             * put 'new' in front
             */

            ++mru.fronts;

            if ( new->prev == NULL )
            {
                return new->name;                 /* should never happen ! */
            }
            else
            {
                if ( new->next == NULL )
                {
                    mru.tail       = new->prev;             /* LAST element */
                    mru.tail->next = NULL;
                }
                else
                {
                    new->prev->next = new->next;
                    new->next->prev = new->prev;

                }
                mru.head->prev = new;
                new->next      = mru.head;
                mru.head       = new;
                new->prev      = NULL;
                return new->name;
            }
        }
        else
        {
            return new->name;
        }
    }
}

/*****************************************************************
** NAME:        NetAddrResolve                              NETMON
** SYNOPSIS:    void NetAddrResolve
**              (
**                u_long  addr,     address in network byte order
**                char   *buf       pointer to buffer
**              )
** RELATED TO:  -
** DESCRIPTION: resolve the address thru DNS
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

int NetAddrResolve( u_long addr, char *buf )
{
    u_long   inetaddr = addr;
    HOSTENT *he;
    INADDR   ia;

    he = gethostbyaddr( (char *) &inetaddr, sizeof( inetaddr ), AF_INET );
    if ( he == NULL )
    {
        ia.s_addr = addr;                         /* address is not known */
        strcpy( buf, (char *)inet_ntoa( &ia ));
        return -1;
    }
    else
    {
        strcpy( buf, he->h_name );
    }
    return 0;
}

/*****************************************************************
** NAME:        PortLookup                                  NETMON
** SYNOPSIS:    void PortLookup
**              (
**                u_short port,     port in network byte order
**                int    *proto     protocol nr
**              )
** RELATED TO:  TcpPortLookUp(), UdpPortLookUp()
** DESCRIPTION: resolve the port thru /etc/services
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

char *PortLookup( register u_short port, int proto )
{
    register short       index;
    register NETPORTENT *cur;
             NETPORTENT *new;
    register NETPORT    *mru;
             char       *protoname;

    /*
     * determine which MRU cache
     */

    index = 0;
    switch ( proto )
    {
        case NET_UDP:
             mru       = &mru_udp;
             protoname = udp_proto;
             break;
        case NET_TCP:
             mru       = &mru_tcp;
             protoname = tcp_proto;
             break;
        default:
             return NULL;
    }

    /*
     * search in MRU port cache

     */

    index = 0;
    for ( cur = mru->head; cur != NULL; cur = cur->next )
    {
        ++index;
        if ( port == cur->port )
            break;
    }

    /*
     * extract or malloc a new element
     */

    if ( cur == NULL )
    {
        /* not found */

        ++mru->misses;

L0:     if ( mru->len >= mru->max )
        {
            /* reuse tail element */

            new       = mru->tail;
            mru->tail = new->prev;
            mru->len--;
        }
        else
        {
            /* allocate new element  */

            new = (NETPORTENT *) malloc( sizeof( NETPORTENT));
            if ( new == NULL )
            {
                if ( mru->len < MRU_TOP )
                    eprintf( "\nMRU_PORT malloc failure\n" );

                mru->max = mru->len;
                goto L0;
            }
            new->port = port;
        }
        if ( mru->len == 0 )
        {
            new->prev       = NULL;               /* add first element */
            new->next       = NULL;
            mru->head       = new;
            mru->tail       = new;
        }
        else
        {
            new->prev       = NULL;
            new->next       = mru->head;
            mru->head->prev = new;
            mru->head       = new;
        }
        mru->len++;
        new->port = port;

        /*
         * resolve the address
         */

        if ( NetPortResolve( port, new->name, protoname ) < 0 )
            ++mru->nores;
        return new->name;
    }
    else
    {
        /* found */

        ++mru->hits;
        new = cur;

        /*
         * Move a found element to the front position only if its too far
         * away. We have set this position to MRU_TOP.
         */

        if ( index > MRU_TOP )
        {
            /*
             * put 'new' in front
             */

            ++mru->fronts;
            if ( new->prev == NULL )
            {
                return new->name;
            }
            else
            {
                if ( new->next == NULL )
                {
                    mru->tail       = new->prev;             /* LAST element */
                    mru->tail->next = NULL;
                }
                else
                {
                    new->prev->next = new->next;
                    new->next->prev = new->prev;

                }
                mru->head->prev = new;
                new->next      = mru->head;
                mru->head       = new;
                new->prev      = NULL;
                return new->name;
            }

        }
        else
        {
            return new->name;
        }
    }
}

/*****************************************************************
** NAME:        NetPortResolve                              NETMON
** SYNOPSIS:    int NetPortResolve
**              (
**                u_short port,     port in network byte order
**                char   *buf,      output buffer
**                char   *proto     protocol name
**              )
** RELATED TO:  PortLookUp()
** DESCRIPTION: resolve the port thru /etc/services
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

int NetPortResolve( u_short port, char *buf, char *proto )
{
    SERVENT *se;

    se = getservbyport( (int) port, proto );
    if ( se == NULL )
    {
        sprintf( buf, "% 5d", ntohs( port ));
        return -1;
    }
    else
    {
        strcpy( buf, se->s_name );
    }
    return 0;
}

/* end of netnames.c */


