/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netterm.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          terminal I/O
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        Terminal I/O control routines
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETTERM_C

#include "netmon.h"
#include "netterm.h"

/*
 * GLOBALS
 */

char *tc_movecursor;
char *tc_clearscreen;

/* dimensions */

int tdim_width;
int tdim_height;

/* special character strings */

char tch_erase;
char tch_kill;

/* terminal capability strings */

char *tcap_clearline;
char *tcap_clearscreen;
char *tcap_movecursor;
char *tcap_cleartoend;
char *tcap_terminit;
char *tcap_termend;
char *tcap_padchar;

/* buffers */

char tbuf_caps[ 1024 ];
char tbuf_string[ 1024 ];
char ts_home[ 32 ];
char ts_lowerleft[ 32 ];
char buf_stdout[ BUFSIZE_STDOUT+1 ];

/* terminal characteristics */

static struct sgttyb term_oldstate;
static struct sgttyb term_newstate;

/* stats */

double termcharcount = 0;

/*
 * local prototypes
 */

/*****************************************************************
** NAME:        TermOpen                                    NETMON
** SYNOPSIS:    void TermOpen
**              (
**                void
**              )
** RELATED TO:  Term..()
** DESCRIPTION: initialise our terminal using termcap info
** REMARKS:     - we use the TERM env variable to determine the
**                type. Preferably an 'xterm'!
**              - exits on any terminal error
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TermOpen( void )
{
    static short  onceonly = 0;
    char         *bufptr;
    char         *padptr;
    char         *mytermname;
    int           status;

    /*
     * skip the once only part
     */

    if ( onceonly )
        goto L0;
    onceonly++;

    /*
     * we won't accept redirection
     */

    if ( ioctl( STDOUT, TIOCGETP, &term_oldstate ) != 0 )
    {
        /* it is no real terminal */
        fprintf( stderr, "\n%s: stdout is redirected, ouch!\n", prog );
        exit( 1 );
    }

    /*
     * read terminal name
     */

    mytermname = getenv( "TERM" );
    if ( mytermname == NULL )
    {
        fprintf( stderr, "\n%s: couldn't find my terminal name.\n", prog );
        exit( 1 );
    }

    /*
     * get termcap entry
     */

    status = tgetent( tbuf_caps, mytermname );
    if ( status != 1 )
    {
        if ( status == -1 )
        {
            fprintf( stderr, "\n%s: couldn't open <%s> termcap file.\n",\
                     prog, mytermname );
        }
        else
        {
            fprintf( stderr, "\n%s: <%s> unknown terminal.\n",\
                     prog, mytermname );
        }
        exit( 1 );
    }

    /*
     * get terminal dimensions
     */

    tdim_width  = tgetnum( "co" );
    tdim_height = tgetnum( "li" );

    /*
     * get term strings
     */

    bufptr = tbuf_string;
    tcap_clearline   = tgetstr( "ce", &bufptr );
    tcap_clearscreen = tgetstr( "cl", &bufptr );
    tcap_movecursor  = tgetstr( "cm", &bufptr );
    tcap_cleartoend  = tgetstr( "cd", &bufptr );
    tcap_terminit    = tgetstr( "ti", &bufptr );
    tcap_termend     = tgetstr( "te", &bufptr );
    tcap_padchar     = tgetstr( "pc", &bufptr );

    /*
     * special cursor movement
     */

    strcpy( ts_home,      tgoto( tcap_movecursor,0,0) );
    strcpy( ts_lowerleft, tgoto( tcap_movecursor,0,tdim_height-1 ));

    /*
     * set stdout to buffered, with large buffer
     */

    setvbuf( stdout, buf_stdout, _IOFBF, BUFSIZE_STDOUT );

    /*
     * set the terminal
     */

    term_newstate = term_oldstate;

    term_newstate.sg_flags |= CBREAK;             /* activate BREAK */
    term_newstate.sg_flags &= ~(ECHO|XTABS);      /* no ECHO and TAB expansion*/
    tch_erase = term_oldstate.sg_erase;
    tch_kill  = term_oldstate.sg_kill;
L0: ioctl( STDOUT, TIOCSETP, &term_newstate );    /* load new state           */

    /*
     * initialise the terminal
     */

    TermPutCap( tcap_terminit );
    TermClear();
}

/*****************************************************************
** NAME:        TermClose                                   NETMON
** SYNOPSIS:    void TermClose
**              (
**                void
**              )
** RELATED TO:  Term..()
** DESCRIPTION: close the terminal and restore the screen
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TermClose( void )
{
    /*
     * reposition the cursor, end terminal
     */

    TermPutCap( ts_lowerleft );
    TermPutCap( tcap_clearline );
    TermPutCap( tcap_termend );

    /*
     * reset the terminal state
     */

    ioctl( STDOUT, TIOCSETP, &term_oldstate );
}

/*****************************************************************
** NAME:        TermClear                                   NETMON
** SYNOPSIS:    void TermClear
**              (
**                void
**              )
** RELATED TO:  Term..()
** DESCRIPTION: clear the terminal screen
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TermClear( void )
{
    TermPutCap( tcap_clearscreen );
}

/*****************************************************************
** NAME:        TermClearToEnd                              NETMON
** SYNOPSIS:    void TermClearToEnd
**              (
**                void
**              )
** RELATED TO:  Term..()
** DESCRIPTION: clear to end of the terminal screen
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TermClearToEnd( void )
{
    TermPutCap( tcap_cleartoend );
}

/*****************************************************************
** NAME:        TermSetXY                                   NETMON
** SYNOPSIS:    int  TermSetXY
**              (
**                int x,      absolute terminal column position
**                int y       absolute terminal row    position
**              )
** RELATED TO:  Term..()
** DESCRIPTION: close the terminal and restore the screen
** REMARKS:     -
** RETURN TYPE: int
** EXIT STATUS: 0 (always)
*****************************************************************/

int  TermSetXY( int x, int y )
{
    tputs( tgoto( tcap_movecursor, x, y ), 1, putstdout );
    return 0;
}

/*****************************************************************
** NAME:        TermSetHome                                 NETMON
** SYNOPSIS:    void TermSetHome
**              (
**                void
**              )
** RELATED TO:  Term..()
** DESCRIPTION: set the cursor home = top left
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TermSetHome( void )
{
    TermPutCap( ts_home );
}

/*****************************************************************
** NAME:        TermPutCap                                  NETMON
** SYNOPSIS:    char *TermPutCap
**              (
**                char *str   ptr to a termcap string
**              )
** RELATED TO:  Term..()
** DESCRIPTION: put a terminal capability
** REMARKS:     - same as TermPrint()
** RETURN TYPE: char *
** EXIT STATUS: ptr to the sent string
*****************************************************************/

char *TermPutCap( char *str )
{
    if ( str == NULL )
        return NULL;
    else
        return (char*) tputs( str, 1 ,putstdout );
}

/*****************************************************************
** NAME:        TermPrint                                   NETMON
** SYNOPSIS:    char *TermPrint
**              (
**                char *str   ptr to a text string
**              )
** RELATED TO:  Term..()
** DESCRIPTION: print a string on your terminal
** REMARKS:     -
** RETURN TYPE: char *
** EXIT STATUS: ptr to the sent string
*****************************************************************/

char *TermPrint( char *string )
{
    if ( string == NULL )
        return NULL;
    else
        return (char*) tputs( string, 1 , putstdout );
}

/*****************************************************************
** NAME:        putstdout                                   NETMON
** SYNOPSIS:    char *putstdout
**              (
**                char c      char to be send to stdout
**              )
** RELATED TO:  TermPrint(), TermPutCap()
** DESCRIPTION: char to std out, used by tputs()
** REMARKS:     - must be hard call, see tputs()
** RETURN TYPE: char *
** EXIT STATUS: ptr to the sent string
*****************************************************************/

void putstdout( char c )
{
    putchar( c );
}

/*****************************************************************
** NAME:        TermValue                                   NETMON
** SYNOPSIS:    void TermValue
**              (
**                MODTEXT *tv,  ptr to MODTEXT array
**                int xoffs,    col origin
**                int yoffs,    row origin
**                int length    # elements in MODTEXT array
**              )
** RELATED TO:  Term..()
** DESCRIPTION: print output on your terminal
** REMARKS:     - only text is currently in use, other
**                output should be generated by using
**                tprintf()
**              - types: only text is supported
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TermValue( MODTEXT *tv, int xoffs, int yoffs, int length )
{
    int   i;
    int   status;

    for( i = 0; i<length; i++ )
    {
        tprintf( xoffs + tv->col, yoffs + tv->row, "%s", (char*) tv->ptr);
        tv++;
    }
}

/*****************************************************************
** NAME:        tprintf                                     NETMON
** SYNOPSIS:    void tprintf
**              (
**                int x,         begin of string column position
**                int y,         begin of string row    position
**                char *format,  printf-styled formatting string
**                ...            the varargs of 'format'
**              )
** RELATED TO:  ...Update()
** DESCRIPTION: replacement for printf(). Used to control your
**              terminal
** REMARKS:     - the format string is the same as in printf()
**              - counts the outputted chars
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void tprintf( int x, int y, char *format, ... )
{
    va_list argp;
    int     cnt;

    va_start( argp, format );
    TermSetXY( x, y );
    cnt = vprintf( format, argp );
    if ( cnt > 0 )
        termcharcount += (double) cnt;
    va_end( argp );
    return;
}

/*****************************************************************
** NAME:        cprintf                                     NETMON
** SYNOPSIS:    void cprintf
**              (
**                char *format,  printf-styled formatting string
**                ...            the varargs of 'format'
**              )
** RELATED TO:  ...Update()
** DESCRIPTION: replacement for printf(), use after tprintf()
** REMARKS:     - the format string is the same as in printf()
**              - counts the outputted chars
** RETURN TYPE: char *
** EXIT STATUS: ptr to the sent string
*****************************************************************/

void cprintf( char *format, ... )
{
    va_list argp;
    int     cnt;

    va_start( argp, format );
    cnt = vprintf( format, argp );
    if ( cnt > 0 )
        termcharcount += (double) cnt;
    va_end( argp );
    return;
}

void eprintf( char *format, ... )
{
    va_list argp;
    va_start( argp, format );

    fflush( stdout );
    TermClear();
    printf( "<%s> encountered the following fatal error:\n\n", prog );
    vprintf( format, argp );
    fflush( stdout );
    va_end( argp );
    sleep( 10 );
    SysClose( 1 );
}
/* end of netterm.c */




